// .png 文件后缀
var suffixPNG = '.png';
var suffixPLIST = '.plist';
var suffixJSON = '.json';
var suffixATLAS = '.atlas';
var suffixMP3 = '.mp3';

// 通过路径和名称拼接PNG文件完整路径
function zGetFullPathPNG(dir, name) {
    return dir + name + suffixPNG;
}

// 新建一个纯色背景
function zNewLayerColorCreate(parent, color, size) {
    var layerColor = new cc.LayerColor(color);
    if (size != undefined) {
        layerColor.setContentSize(size);
    }
    parent.addChild(layerColor);
    return layerColor;
}

// 新建一个精灵
function zNewSpriteCreate(dir, name, parent) {
    var sprite = new cc.Sprite(dir + name + suffixPNG);
    parent.addChild(sprite);
    return sprite;
}

// 新建粒子
function zNewParticleCreate(dir, name, parent) {
    // zLog(dir + name + suffixPLIST);
    var particle = new cc.ParticleSystem(dir + name + suffixPLIST);
    parent.addChild(particle);
    return particle;
}

function zAddParticleDone(obj) {
    // ddPlayEffect('effect', 'res/sound/effect/flash_star.mp3', false);
    var particle = zNewParticleCreate('res/particle/', 'star', obj);
    particle.setPosition(obj.convertToNodeSpace(cc.p(cc.winSize.width / 2, cc.winSize.height / 2)));
    particle.scheduleOnce(() => {
        particle.stopSystem();
    }, 3);
    return particle;
}

// 新建spine动画
function zNewSpineAnimationCreate(dir, name, parent) {
    var spine = sp.SkeletonAnimation.createWithJsonFile(dir + name + suffixJSON, dir + name + suffixATLAS);
    parent.addChild(spine);
    return spine;
}

// 新建遮罩节点
function zNewClippingNodeCreate(parent, inverted, alphaThreshold, stencilDir, stencilName) {
    var maskNode = new cc.ClippingNode();
    parent.addChild(maskNode);
    maskNode.setInverted(inverted);
    maskNode.setAlphaThreshold(alphaThreshold);
    maskNode.setStencil(new cc.Sprite(stencilDir + stencilName + suffixPNG));
    return maskNode;
}

// 新建遮罩节点
function zNewClippingNodeCreate2(parent, inverted, alphaThreshold) {
    var maskNode = new cc.ClippingNode();
    parent.addChild(maskNode);
    maskNode.setInverted(inverted);
    maskNode.setAlphaThreshold(alphaThreshold);
    return maskNode;
}

// 添加一组精灵
function zAddRes(dir, arrName, parent) {
    var res = [];
    for (var i = 0; i < arrName.length; ++i) {
        res.push(zNewSpriteCreate(dir, arrName[i], parent));
        // zAddTouchLogPosEvent(res[i]);
    }
    return res;
}

// 设置精灵纹理
function zSetSpriteTexture(dir, name, sprite) {
    // zLog(dir + name + suffixPNG);
    sprite.setTexture(dir + name + suffixPNG);
}

// 设置精灵事件
function zSetSpriteEvent(listener, sprite) {
    cc.eventManager.addListener(listener, sprite);
}

// 设置节点事件
function zSetNodeEvent(listener, node) {
    cc.eventManager.addListener(listener, node);
}

// 暂停事件
function zPauseTargetEvent(target, recursion) {
    cc.eventManager.pauseTarget(target, recursion ? recursion : false);
}

// 恢复事件
function zResumeTargetEvent(target, recursion) {
    cc.eventManager.resumeTarget(target, recursion ? recursion : false);
}

// 新建 label
function zNewLabelCreate(text, front, frontSize, parent) {
    var label = new cc.LabelTTF(text, front ? front : 'arial', frontSize);
    parent.addChild(label);
    return label;
}

// 新建带事件的精灵
function zNewSpriteEventCreate(dir, name, parent, cbTouchBegin, cbTouchMoved, cbTouchEnded, cbTouchCancel) {
    var sprite = zNewSpriteCreate(dir, name, parent);
    var listener = zNewEventListenerCreate(cbTouchBegin, cbTouchMoved, cbTouchEnded, cbTouchCancel);
    cc.eventManager.addListener(listener, sprite);
    return sprite;
}

// 改变父节点
function zChangeParent(child, newParent) {
    child.retain();
    child.removeFromParent(false);
    newParent.addChild(child);
    child.release();
}

// 新建事件监听器
function zNewEventListenerCreate(cbTouchBegin, cbTouchMoved, cbTouchEnded, cbTouchCancel) {
    var listener = cc.EventListener.create({
        event: cc.EventListener.TOUCH_ONE_BY_ONE,
        swallowTouches: true,
        target: this,
        onTouchBegan: function (touch, event) {
            var target = event.getCurrentTarget();
            var posInNode = target.convertToNodeSpace(touch.getLocation());
            var size = target.getContentSize();
            var rect = cc.rect(0, 0, size.width, size.height);
            if (!(cc.rectContainsPoint(rect, posInNode))) {
                return false;
            }
            if (cbTouchBegin) {
                cbTouchBegin(touch, target);
            }
            return true;                    // 很重要 不加就不响应后面的 onTouchMoved onTouchEnded onTouchCancelled 事件
        },
        onTouchMoved: function (touch, event) {
            var target = event.getCurrentTarget();
            if (cbTouchMoved) {
                cbTouchMoved(touch, target);
            }
            // return true;
        },
        onTouchEnded: function (touch, event) {
            var target = event.getCurrentTarget();
            var posInNode = target.convertToNodeSpace(touch.getLocation());
            var size = target.getContentSize();
            var rect = cc.rect(0, 0, size.width, size.height);
            if (!(cc.rectContainsPoint(rect, posInNode))) {
                if (cbTouchCancel) {
                    cbTouchCancel(touch, target);
                }
                return false;
            }
            if (cbTouchEnded) {
                cbTouchEnded(touch, target);
            }
            return true;
        }
    });
    return listener;
}

function zAddKeyboardEventCreate(cbKeyPressed, cbKeyReleased, target) {
    cc.eventManager.addListener({
        event: cc.EventListener.KEYBOARD,
        onKeyPressed: function (keyCode, event) {
            if (cbKeyPressed) {
                cbKeyPressed(keyCode, event);
            }
        },
        onKeyReleased: function (keyCode, event) {
            if (cbKeyReleased) {
                cbKeyReleased(keyCode, event);
            }
        }
    }, target);
}

function zAddKeyboardBackEventCreate(target) {
    var cbKeyPressed = function (keyCode, event) {
        if (keyCode == cc.KEY.back) {
            cc.director.popScene();
        }
    };
    zAddKeyboardEventCreate(cbKeyPressed, null, target);
}

// 获取节点的世界坐标
function zGetWorldPosition(node) {
    return node.getParent().convertToWorldSpace(node.getPosition());
}

// 获取本地坐标
function zGetNodePosition(node, worldPos) {
    return node.convertToNodeSpace(worldPos);
}

function zGetBezierTo(time, p1, p2) {
    var bezier = [p1, cc.p((p1.x + p2.x) / 2, Math.max(p1.y, p2.y) + 100), p2];
    return cc.bezierTo(time, bezier);
}

function zGetBezierTo2(time, p1, p2) {
    var bezier = [p1, cc.p((p1.x + p2.x) / 2 + 200, Math.max(p1.y, p2.y)), p2]
    return cc.bezierTo(time, bezier);
}

Array.prototype.top = function () {
    if (this.length > 0) {
        return this[this.length - 1];
    }
    return null;
};
Array.prototype.first = function () {
    if (this.length > 0) {
        return this[0];
    }
    return null;
};