var Global_DP_WhackAMole = {
    dir: 'res/game/whack_a_mole/',
    cavePos: [
        cc.p(-150.17, 552.52), cc.p(229.91, 557.13), cc.p(-347.51, 403.67), cc.p(-0.98, 394.33),
        cc.p(313.47, 411.99), cc.p(-265.83, 247.83), cc.p(211.39, 237.97)
    ],
    coinBox: null,
    timeClock: null,
    hammer: null,
    mouseManager: null,
    layer: null
};

var DP_WhackAMole_MouseManager = cc.Node.extend({
    gameOver: null,
    curMouseCount: 0,
    curFruitCount: 0,
    curMineCount: 0,
    score: 0,
    difficulty: 0,
    mouseBaseLifeTime: 1,
    mouseDeltaLifeTime: 1,
    mouseBaseRandomTime: 0.5,
    mouseDeltaRandomTime: 1,
    cavePosIdx: [0, 1, 2, 3, 4, 5, 6],
    timeOutId_randomMouse: null,
    cave_all: null,
    cave_used: null,
    cave_unused: null,
    continuous_counting: 0,
    ctor: function () {
        this._super();

        this.cave_unused = [0, 1, 2, 3, 4, 5, 6];
        this.cave_used = [];
    },
    setCaveAll: function (cave_all) {
        // 设置全部地鼠洞
        this.cave_all = cave_all;
    },
    getUnusedCave: function () {
        // 随机出可用地鼠洞索引
        var cave_unused = this.cave_unused.splice(Math.floor(Math.random() * this.cave_unused.length), 1);
        if (cave_unused.length > 0) {
            this.cave_used.push(cave_unused);
            // zLog('cave_used : ', this.cave_used);
            // 可用地鼠洞记录索引
            this.cave_all[cave_unused].cave_unused = cave_unused;
            // 返回可用地鼠洞
            return this.cave_all[cave_unused];
        }
        return null;
    },
    pushUnusedCave: function (cave_unused) {
        // 从已使用的地鼠洞索引中找到不再使用的
        this.cave_used.splice(this.cave_used.indexOf(cave_unused), 1);
        // 把不再使用的索引放入未使用的数组中
        this.cave_unused.push(cave_unused);
    },
    randomMouse2: function () {
        if (this.gameOver) {
            return;
        }
        // 找一个未使用的地鼠洞
        var cave = this.getUnusedCave();
        if (cave) {
            // 地鼠洞新建一个地鼠
            cave.createMouse();
            // 地鼠延时自动移除
            cave.autoRemoveMouse();
        }
        // 根据当前地鼠数决定下次地鼠怎么出现
        // var timeNext = (this.mouseBaseRandomTime + Math.random() * this.mouseDeltaRandomTime) * 0.3 + this.curMouseCount * Math.random();
        var timeNext = this.mouseBaseRandomTime + this.curMouseCount * Math.random();
        // zLog('timeNext : ', timeNext.toFixed(2));
        // zLog('------');
        this.scheduleOnce(this.randomMouse2.bind(this), timeNext / 2);
    }
});

// 地鼠
var DP_WhackAMole_Mouse = cc.Node.extend({
    isBeHit: false,
    ctor: function (difficulty) {
        this._super();
        var self = this;
        var name = Math.random() * 3 < 2 ? 'dishu' : 'zhadan';
        var spine = zNewSpineAnimationCreate(Global_DP_WhackAMole.dir + 'spine/', 'skeleton', this);
        // spine.setAnimation(0, name + 'shang', false);
        spine.setContentSize(150, 200);
        spine.setPosition(150 / 2, 120 / 2);
        spine.setTimeScale(1.7 + difficulty / 3);
        spine.setCompleteListener((trackEntry) => {
            var aniName = trackEntry.animation ? trackEntry.animation.name : '';
            if (aniName.indexOf('beiza') >= 0) {
                self.removeSelf();
                /*if (name == 'dishu') {
                    self.removeSelf();
                } else if (name == 'zhadan') {
                    self.removeSelf();
                }*/
            } else if (aniName.indexOf('xia') >= 0) {
                self.removeSelf();
            }
        });
        this.setName(name);
        this.setContentSize(150, 200);
        this.setAnchorPoint(0.5, 0.2);
        this.spine = spine;
        // zShowContentSize(this);

        var listener = zNewEventListenerCreate(function (touch, target) {
            // zLog(target.getName());
            zPauseTargetEvent(target, true);
            var hammer = Global_DP_WhackAMole.hammer;
            var time = hammer.hit(cc.pAdd(target.getPosition(), cc.p(200, 110)), target.getLocalZOrder() + 1);
            target.scheduleOnce(target.beHit, time);
            target.unschedule(target.autoRemoveSelf);
        }, null, null, null);
        zSetNodeEvent(listener, this);
    },
    cleanup: function () {
        this._super();
        // zLog('remove : ', this.getName());
    },
    up: function () {
        this.spine.setAnimation(0, this.getName() + 'shang', false);
    },
    down: function () {
        this.spine.setAnimation(0, this.getName() + 'xia', false);
        zPauseTargetEvent(this, true);
    },
    beHit: function () {
        this.unschedule(this.down);
        this.spine.setAnimation(0, this.getName() + 'beiza', false);
        this.isBeHit = true;
        if (this.getName() == 'dishu') {
            var coinBox = Global_DP_WhackAMole.coinBox;
            coinBox.runGetCoinAction(cc.pAdd(zGetNodePosition(coinBox, zGetWorldPosition(this)), cc.p(0, 110)), 1);
            ddSound.playEffect('game_whack_a_mole_dizzy');
        } else {
            ddSound.playEffect('game_whack_a_mole_bomb');
        }
    },
    removeSelf: function () {
        var mouseManager = Global_DP_WhackAMole.mouseManager;
        this.runAction(cc.removeSelf(true));
        // this.removeFromParent(true);
        this.cave.mouse = null;
        mouseManager.curMouseCount--;
        mouseManager.pushUnusedCave(this.cave.cave_unused);
    },
    autoRemoveSelf: function (time) {
        var self = this;
        self.unschedule(self.down);
        self.scheduleOnce(self.down, time);
    }
});

var DP_WhackAMoleLayer = cc.Layer.extend({
    ctor: function () {
        this._super();
        this.x = cc.winSize.width / 2;
        this.y = Global_DP.sinkY;
        this.setName('WhackAMoleLayer');
        Global_DP_WhackAMole.layer = this;

        this.addBG();
        this.addMouseManager();
        this.addCave();
        this.addTimeClock();
        this.addCoinBox();
        this.addHammer();

        zAddBackButtonBegin(this);
        // this.addBackBtn();
    },
    onEnter: function () {
        this._super();
        // this.count_down.count();
        zPauseTargetEvent(this, true);
        this.countDown = function () {
            this.count_down.count();
            zResumeTargetEvent(this, true);
            var coinBox = Global_DP_WhackAMole.coinBox;
            coinBox.setCoinNumber(zGetLocalCoinNumber());
        };
        zAddInsertCoinsLayer(this, this.countDown);
    },
    cleanup: function () {
        this._super();
    },
    addBG: function () {
        var bg_hd = zNewSpriteCreate(Global_DP_WhackAMole.dir, 'bg_hd', this);
        bg_hd.setAnchorPoint(0.5, 0);
        bg_hd.setPosition(0, 0);
    },
    addCave: function () {
        var self = this;
        var cnt = 7;
        var dir = Global_DP_WhackAMole.dir;

        var caves = [];
        for (var i = 0; i < cnt; ++i) {
            var cave = zNewSpriteCreate(dir, 'cave', this);
            var cave_side = zNewSpriteCreate(dir, 'cave_side', cave);
            var cave_mask = zNewSpriteCreate(dir, 'cave_stencil', this);
            // var label = zNewLabelCreate(cave.getLocalZOrder().toString(), null, 40, cave);
            cave.setPosition(Global_DP_WhackAMole.cavePos[i]);
            cave.setLocalZOrder(10 * i);
            cave_side.setPosition(122.60, 16.41);
            cave_mask.setPosition(cc.pAdd(cave.getPosition(), cc.p(6.37, -160.18)));
            cave_mask.setLocalZOrder(cave.getLocalZOrder() + 2);
            // label.setPosition(cave.width / 2, cave.height / 2);
            // zAddTouchLogPosEvent(cave);
            // zAddTouchLogPosEvent(cave_side);
            // zAddTouchLogPosEvent(cave_mask);

            (function f(cave, idx) {
                var mouseManager = Global_DP_WhackAMole.mouseManager;
                cave.createMouse = function () {
                    var mouse = new DP_WhackAMole_Mouse(mouseManager.difficulty);
                    mouse.setPosition(cave.getPosition());
                    mouse.up();
                    mouse.cave = cave;
                    cave.mouse = mouse;
                    self.addChild(mouse, cave.getLocalZOrder() + 1);
                    mouseManager.curMouseCount++;
                };
                cave.autoRemoveMouse = function () {
                    // var timeLife = (Math.random() * mouseManager.mouseDeltaLifeTime + mouseManager.mouseBaseLifeTime) * 0.3 + (cnt - mouseManager.curMouseCount) * 0.2;
                    var timeLife = Math.random() * mouseManager.mouseDeltaLifeTime / 2 + mouseManager.mouseBaseLifeTime + (cnt - mouseManager.curMouseCount) * Math.random() * 0.1;
                    // zLog('timeLife : ', timeLife.toFixed(2));
                    // cave.mouse.scheduleOnce(cave.mouse.autoRemoveSelf, timeLife);
                    var mouse = cave.mouse;
                    mouse.autoRemoveSelf(timeLife);
                }
            })(cave, i);
            caves.push(cave);
            // this.cave.push(cave);
        }

        this.mouseManager.setCaveAll(caves);

        // this.mouseManager.randomMouse2();
    },
    addMouseManager: function () {
        this.mouseManager = new DP_WhackAMole_MouseManager();
        this.addChild(this.mouseManager);
        Global_DP_WhackAMole.mouseManager = this.mouseManager;
    },
    addTimeClock: function () {
        var timeTotal = 60;
        var self = this;
        var time_out_cb = function () {
            // zLog('time out');
            var mouseManager = Global_DP_WhackAMole.mouseManager;
            mouseManager.gameOver = true;
            zAddAwardCoinLayer(self, mouseManager.score);
            umengEvent('GAME_SRC_DONE', 'whack_a_mole');
        };
        var timeClock = zAddMyTimeClock(this, timeTotal, time_out_cb);
        // timeClock.rewardTime();

        Global_DP_WhackAMole.timeClock = timeClock;

        var count_down_0_cb = function () {
            ddSound.stopAll();
            ddSound.playEffect('game_whack_a_mole_find_mouse');
            timeClock.startTiming();
            self.mouseManager.randomMouse2();
        };
        this.count_down = zAddCountDown(this, 1, 3, count_down_0_cb);

    },
    addCoinBox: function () {
        var coinBox = zAddCoinBox(this);
        coinBox.addScore = function (score) {
            var mouseManager = Global_DP_WhackAMole.mouseManager;
            var difficulty = mouseManager.difficulty;
            if (mouseManager.gameOver) {
                return;
            }
            coinBox.setCoinNumber(coinBox.getCoinNumber() + score);
            mouseManager.score += score;
            if (mouseManager.score == 5 * (difficulty + (difficulty + 1))) {
                // zLog('score -> difficulty : ', mouseManager.difficulty);
                var timeClock = Global_DP_WhackAMole.timeClock;
                timeClock.rewardTime(5);
                // timeClock.setCurTime(timeClock.getCurTime() + 5);
                mouseManager.mouseBaseLifeTime /= 1.2;
                mouseManager.mouseDeltaLifeTime /= 1.2;
                mouseManager.mouseBaseRandomTime /= 1.1;
                mouseManager.mouseDeltaRandomTime /= 1.1;
                difficulty += 1;
                // zLog('**** add difficulty ****');
                mouseManager.difficulty = difficulty;
            }
        };
        Global_DP_WhackAMole.coinBox = coinBox;
        // zShowContentSize(this.coinBox);
    },
    addHammer: function () {
        var hammer = zNewSpriteCreate(Global_DP_WhackAMole.dir, 'hammer', this);
        hammer.setLocalZOrder(20000);
        hammer.setVisible(false);
        hammer.setAnchorPoint(156 / hammer.width, 18 / hammer.height);
        hammer.hit = function (pos, zOrder) {
            var time = 0.05;
            hammer.setLocalZOrder(zOrder);
            hammer.setPosition(pos);
            hammer.stopAllActions();
            hammer.setRotation(30);
            hammer.setOpacity(255);
            hammer.runAction(cc.sequence(
                cc.show(),
                cc.spawn(cc.rotateTo(time, -30), cc.bezierBy(time, [cc.p(0, 0), cc.p(-40, 10), cc.p(-80, -20)])).easing(cc.easeIn(2)),
                cc.delayTime(0.2),
                cc.fadeOut(0.2),
                cc.hide()
            ));
            ddSound.playEffect('game_whack_a_mole_hit');
            return 0.05;
        };
        Global_DP_WhackAMole.hammer = hammer;
    }
});
var DP_WhackAMoleScene = cc.Scene.extend({
    ctor: function () {
        this._super();

        var layer = new DP_WhackAMoleLayer();
        this.addChild(layer);

        zAddKeyboardBackEventCreate(this);
    }
});